import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Analyzer extends JDialog implements Runnable
{
  private SquareCell squareCell;
  private byte cells[][];
  private int rows;
  private int columns;
  
  private JPanel panel;
  private JLabel descriptionLabel;
  private JLabel periodLabel;
  private JLabel xDisplacementLabel;
  private JLabel yDisplacementLabel;

  private int r1;
  private int c1;
  private int rn;
  private int cn;

  private boolean clear;

  private byte pattern[][];
  private int patternR1;
  private int patternC1;
  private int patternRn;
  private int patternCn;

  private int period;

  public Analyzer(JFrame owner, SquareCell squareCell)
  {
    super(owner, "Analyzer", true);
    this.squareCell = squareCell;
    panel = new JPanel();
    descriptionLabel = new JLabel("Pattern is an Oscillator");
    periodLabel = new JLabel("Period ");
    xDisplacementLabel = new JLabel("X Displacement ");
    yDisplacementLabel = new JLabel("Y Displacement ");
    panel.setBorder(BorderFactory.createLineBorder(getBackground(), 15));
    panel.setLayout(new GridLayout(4, 1, 5, 5));
    panel.add(descriptionLabel);
    panel.add(periodLabel);
    panel.add(xDisplacementLabel);
    panel.add(yDisplacementLabel);
    setLayout(new BorderLayout());
    add(panel, BorderLayout.NORTH);
    pack();
    setResizable(false);
  }

  private void getPatternLimits()
  {
    r1 = rows;
    c1 = columns;
    rn = 0;
    cn = 0;
    clear = true;
    for (int r = 0; r < rows; r++)
      for (int c = 0; c < columns; c++)
        if (cells[r][c] > 0)
        {
          clear = false;
          if (r < r1)
            r1 = r;
          if (r > rn)
            rn = r;
          if (c < c1)
            c1 = c;
          if (c > cn)
            cn = c;
        }
  }

  public boolean save(byte cells[][], int rows, int columns)
  {
    this.cells = cells;
    this.rows = rows;
    this.columns = columns;
    getPatternLimits();
    if (clear)
      return true;
    pattern = new byte[rn - r1 + 1][cn - c1 + 1]; 
    for (int r = r1; r <= rn; r++)
      for (int c = c1; c <= cn; c++)
        pattern[r - r1][c - c1] = cells[r][c];
    patternR1 = r1;
    patternC1 = c1;
    patternRn = rn;
    patternCn = cn;
    period = 0;
    return false;
  }

  public boolean compare()
  {
    getPatternLimits();
    period++;
    if (((patternRn - patternR1) != (rn - r1)) || ((patternCn - patternC1) != (cn - c1)))
      return false;
    for (int r = r1; r <= rn; r++)
      for (int c = c1; c <= cn; c++)
      {
        if (cells[r][c] != pattern[r - r1][c - c1])
          return false;
      }
    if ((r1 == patternR1) && (rn == patternRn) && (c1 == patternC1) && (cn == patternCn))
    {
      if (period == 1)
        descriptionLabel.setText("Pattern is Stable");
      else
        descriptionLabel.setText("Pattern is an Oscillator");
    }
    else
      descriptionLabel.setText("Pattern is a Ship");
    periodLabel.setText("Period " + period);
    xDisplacementLabel.setText("X Displacement " + (c1 - patternC1));
    yDisplacementLabel.setText("Y Displacement " + (r1 - patternR1));
    return true;
  }

  public void run()
  {
    cells = squareCell.analyzerStep();
    while (! compare())
    {
      if (! squareCell.isAnalyzing())
      {
        squareCell.drawDisplay();
        return;
      }
      cells = squareCell.analyzerStep();
    }
    squareCell.showAnalysisResults();
  }
}
