import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class ColorSelector extends JDialog implements ActionListener
{
  private final int noColors = 27;
  private final int noStates = 25;
  private final int grid = 25;
  private final int selector = 26;

  private JFrame owner;
  private SquareCell squareCell;

  private Color colors[];
  private File currentDirectory;

  private ColorPanel colorPanels[];
  private JLabel displayLabel;
  private JButton defaultButton;
  private JButton loadButton;
  private JButton saveButton;

  public ColorSelector(JFrame owner, SquareCell squareCell, int currentState)
  {
    super(owner, "Color Selector", false);
    this.owner = owner;
    this.squareCell = squareCell;
    colors = new Color[noColors];
    setDefaultColors();
    colorPanels = new ColorPanel[noColors];
    JPanel gridPanel = new JPanel();
    gridPanel.setLayout(new GridLayout(3, noColors / 3, 5, 5));
    for (int c = 0; c < noColors; c++)
    {
      colorPanels[c] = new ColorPanel(owner, this, colors[c], c);
      gridPanel.add(colorPanels[c]);
    }
    displayLabel = new JLabel("State " + currentState + " selected");
    JPanel displayPanel = new JPanel();
    displayPanel.add(displayLabel);
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new BorderLayout(0, 5));
    controlPanel.add(gridPanel, BorderLayout.NORTH);
    controlPanel.add(displayPanel, BorderLayout.SOUTH);
    defaultButton = new JButton("Default");
    defaultButton.addActionListener(this);
    loadButton = new JButton("Load");
    loadButton.addActionListener(this);
    saveButton = new JButton("Save");
    saveButton.addActionListener(this);
    JPanel buttonPanel = new JPanel();
    buttonPanel.add(defaultButton);
    buttonPanel.add(loadButton);
    buttonPanel.add(saveButton);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout(0, 10));
    basePanel.add(controlPanel, BorderLayout.NORTH);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }

  private void setDefaultColors()
  {
    colors[0] = Color.black;
    colors[1] = Color.red;
    colors[2] = Color.green;
    colors[3] = Color.blue;
    colors[4] = Color.yellow;
    colors[5] = Color.cyan;
    colors[6] = Color.magenta;
    colors[7] = new Color(92, 36, 110);
    colors[8] = new Color(0, 99, 140);
    colors[9] = new Color(176, 196, 222);
    colors[10] = new Color(115, 61, 26);
    colors[11] = new Color(189, 183, 107);
    colors[12] = new Color(205, 133, 63);
    colors[13] = new Color(47, 79, 79);
    colors[14] = new Color(255, 128, 0);
    colors[15] = new Color(255, 215, 0);
    colors[16] = new Color(184, 134, 11);
    colors[17] = new Color(0, 100, 0);
    colors[18] = new Color(173, 255, 47);
    colors[19] = new Color(107, 142, 35);
    colors[20] = new Color(0, 250, 154);
    colors[21] = new Color(72, 209, 204);
    colors[22] = new Color(0, 0, 128);
    colors[23] = new Color(132, 112, 255);
    colors[24] = new Color(160, 32, 240);
    colors[25] = new Color(51, 51, 51);
    colors[26] = new Color(102, 102, 102);
  }

  private void writeColors()
  {
    JFileChooser fc = new JFileChooser();
    fc.setCurrentDirectory(currentDirectory);
    if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
    {
      try
      {
        File f = fc.getSelectedFile();
        currentDirectory = fc.getCurrentDirectory();
        PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(f)));
        for (int c = 0; c < noColors; c++)
        {
          int red = colors[c].getRed();
          int green = colors[c].getGreen();
          int blue = colors[c].getBlue();
          pw.println("r = " + red + ", g = " + green + ", b = " + blue);
        }
        pw.close();
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }
    }
  }

  private void readColors()
  {
    JFileChooser fc = new JFileChooser();
    fc.setCurrentDirectory(currentDirectory);
    if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
    {
      try
      {
        File f = fc.getSelectedFile();
        currentDirectory = fc.getCurrentDirectory();
        BufferedReader br = new BufferedReader(new FileReader(f));
        for (int c = 0; c < noColors; c++)
        {
          String s = br.readLine();
          int red = new Integer(s.substring(4, s.indexOf(", g = ")).trim());
          s = s.substring(s.indexOf("g = "));
          int green = new Integer(s.substring(4, s.indexOf(", b = ")).trim());
          s = s.substring(s.indexOf("b = "));
          int blue = new Integer(s.substring(4).trim());
          colors[c] = new Color(red, green, blue);
          colorPanels[c].setBackground(colors[c]);
        }
        squareCell.drawDisplay();
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }
    }
  }

  public Color getColor(int state)
  {
    if ((state >= 0) && (state < noStates))
      return colors[state];
    else
      return Color.black;
  }

  public Color getGridColor()
  {
    return colors[grid];
  }

  public Color getSelectorColor()
  {
    return colors[selector];
  }

  public void setColor(Color color, int c)
  {
    colors[c] = color;
    squareCell.drawDisplay();
  }

  public void setCurrentState(int c)
  {
    if (c < squareCell.getNoStates())
    {
      squareCell.setCurrentState(c);
      setIdentifier(c);
    }
  }

  public void setIdentifier(int c)
  {
    String s;
    if (c == grid)
      s = "Grid";
    else if (c == selector)
      s = "Selector";
    else
      s = "State " + c;
    displayLabel.setText(s + " color    State " + squareCell.getCurrentState() + " selected");
  }

  public void hideIdentifier()
  {
    displayLabel.setText("State " + squareCell.getCurrentState() + " selected");
  }

  public void showCurrentState()
  {
    hideIdentifier();
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == defaultButton)
    {
      setDefaultColors();
      for (int c = 0; c < noColors; c++)
        colorPanels[c].setBackground(colors[c]);
      squareCell.drawDisplay();
    }
    else if (source == loadButton)
    {
      readColors();
    }
    else if (source == saveButton)
    {
      writeColors();
    }
  }
}
