import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class NewWorld extends JDialog implements ActionListener, DocumentListener
{
  private int rows;
  private int columns;

  private boolean changed;

  private JLabel rowsLabel;
  private JLabel columnsLabel;
  private JTextField rowsInput;
  private JTextField columnsInput;
  private JButton okButton;
  private JButton cancelButton;

  public NewWorld(JFrame owner, int rows, int columns)
  {
    super(owner, "New World", true);
    this.rows = rows;
    this.columns = columns;
    rowsLabel = new JLabel("Rows");
    columnsLabel = new JLabel("Columns");
    rowsInput = new JTextField("" + rows, 4);
    rowsInput.getDocument().addDocumentListener(this);
    columnsInput = new JTextField("" + columns, 4);
    columnsInput.getDocument().addDocumentListener(this);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel worldSizePanel = new JPanel();
    worldSizePanel.setLayout(new GridLayout(2, 2, 5, 5));
    JPanel rowsInputPanel = new JPanel();
    rowsInputPanel.add(rowsInput);
    JPanel columnsInputPanel = new JPanel();
    columnsInputPanel.add(columnsInput);
    worldSizePanel.add(rowsLabel);
    worldSizePanel.add(rowsInputPanel);
    worldSizePanel.add(columnsLabel);
    worldSizePanel.add(columnsInputPanel);
    worldSizePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "World Size"));
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(worldSizePanel, BorderLayout.NORTH);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
    setValues();
  }

  private void setValues()
  {
    try
    {
      int r = new Integer(rowsInput.getText().trim());
      int c = new Integer(columnsInput.getText().trim());
      rows = r;
      if (rows < 10)
        rows = 10;
      columns = c;
      if (columns < 10)
        columns = 10;
    }
    catch (Exception e)
    {
    }
  }

  public int getRows()
  {
    return rows;
  }

  public int getColumns()
  {
    return columns;
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
