import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class Primordia extends Rule
{
  private int noStates = 2;
  private int K = 3;
  private int L = 0;
  private int M = 1;
  private int N = 0;
  private boolean doK = true;
  private boolean doL = true;
  private boolean doM = true;
  private boolean doN = true;
  
  public Primordia(JFrame mainFrame, SquareCell squareCell)
  {
    super(mainFrame, squareCell);
    this.mainFrame = mainFrame;
    this.squareCell = squareCell;
  }

  public void setNewRule()
  {
    Random generator = new Random();
    if (doK)
    {
      K = generator.nextInt(26);
    }
    if (doL)
    {
      L = generator.nextInt(26);
    }
    if (doM)
    {
      M = generator.nextInt(26);
    }
    if (doN)
    {
      N = generator.nextInt(26);
    }
  }

  public void setDefault()
  {
  }

  public void setOptions()
  {
    PrimordiaOptionsDialog primordiaOptionsDialog = new PrimordiaOptionsDialog(mainFrame);
    primordiaOptionsDialog.setData(this);
    centerDialog(primordiaOptionsDialog);
    primordiaOptionsDialog.setVisible(true);
    if (primordiaOptionsDialog.hasChanged())
      primordiaOptionsDialog.getData(this);
  }

  public void changeRule()
  {
    PrimordiaDialog primordiaDialog = new PrimordiaDialog(mainFrame, this);
    centerDialog(primordiaDialog);
    primordiaDialog.setVisible(true);
    if (primordiaDialog.hasChanged())
      primordiaDialog.getData(this);
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  public void writeRule(PrintWriter pw)
  {
    String str;
    str = "#States = " + noStates;
    pw.println(str);
    str = "#K = " + K;
    pw.println(str);
    str = "#L = " + L;
    pw.println(str);
    str = "#M = " + M;
    pw.println(str);
    str = "#N = " + N;
    pw.println(str);
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      String value;
      value = br.readLine().substring(10);
      noStates = new Integer(value.trim());
      value = br.readLine().substring(5);
      K = new Integer(value.trim());
      value = br.readLine().substring(5);
      L = new Integer(value.trim());
      value = br.readLine().substring(5);
      M = new Integer(value.trim());
      value = br.readLine().substring(5);
      N = new Integer(value.trim());
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public int step(int r, int c)
  {
      int cell = squareCell.getNeighbor(r, c);
      int count =
        squareCell.getNeighbor(r - 1, c - 1) +
        squareCell.getNeighbor(r - 1, c    ) +
        squareCell.getNeighbor(r - 1, c + 1) +
        squareCell.getNeighbor(r    , c - 1) +
        squareCell.getNeighbor(r    , c + 1) +
        squareCell.getNeighbor(r + 1, c - 1) +
        squareCell.getNeighbor(r + 1, c    ) +
        squareCell.getNeighbor(r + 1, c + 1);
			if ((count < K - M) || (count > K + L + N))
			  cell--;
			else if ((count >= K) && (count <= K + L))
			  cell++;
			if (cell < 0)
			  return 0;
			if (cell > noStates - 1)
			  return noStates - 1;
			return cell;
  }

  public void setNoStates(int value)
  {
    noStates = value;
  }

  public int getNoStates()
  {
    return noStates;
  }

  public int getK()
  {
    return K;
  }

  public int getL()
  {
    return L;
  }

  public int getM()
  {
    return M;
  }

  public int getN()
  {
    return N;
  }

  public boolean getDoK()
  {
    return doK;
  }

  public boolean getDoL()
  {
    return doL;
  }

  public boolean getDoM()
  {
    return doM;
  }

  public boolean getDoN()
  {
    return doN;
  }

  public void setK(int value)
  {
    K = value;
  }

  public void setL(int value)
  {
    L = value;
  }

  public void setM(int value)
  {
    M = value;
  }

  public void setN(int value)
  {
    N = value;
  }

  public void setDoK(boolean value)
  {
    doK = value;
  }

  public void setDoL(boolean value)
  {
    doL = value;
  }

  public void setDoM(boolean value)
  {
    doM = value;
  }

  public void setDoN(boolean value)
  {
    doN = value;
  }
}
