import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class PrimordiaOptionsDialog extends JDialog implements ActionListener, DocumentListener
{
  private int noStates;
  private boolean changed;

  private JLabel noStatesLabel;
  private JTextField noStatesInput;
  private JCheckBox doKButton;
  private JCheckBox doLButton;
  private JCheckBox doMButton;
  private JCheckBox doNButton;
  private JButton okButton;
  private JButton cancelButton;

  public PrimordiaOptionsDialog(JFrame owner)
  {
    super(owner, "Options", true);
    noStatesLabel = new JLabel("No States");
    noStatesInput = new JTextField("" + noStates, 3);
    noStatesInput.getDocument().addDocumentListener(this);
    JPanel noStatesPanel = new JPanel();
    noStatesPanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel noStatesInputPanel = new JPanel();
    noStatesInputPanel.add(noStatesInput);
    noStatesPanel.add(noStatesLabel);
    noStatesPanel.add(noStatesInputPanel);
    noStatesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ""));
    doKButton = new JCheckBox("Do K");
    doKButton.setSelected(true);
    doLButton = new JCheckBox("Do L");
    doLButton.setSelected(true);
    doMButton = new JCheckBox("Do M");
    doMButton.setSelected(true);
    doNButton = new JCheckBox("Do N");
    doNButton.setSelected(true);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new GridLayout(4, 1, 5, 5));
    controlPanel.add(doKButton);
    controlPanel.add(doLButton);
    controlPanel.add(doMButton);
    controlPanel.add(doNButton);
    controlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "New Rule Options"));
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(noStatesPanel, BorderLayout.NORTH);
    basePanel.add(controlPanel, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  public void setData(Primordia rule)
  {
    noStates = rule.getNoStates();
    noStatesInput.setText("" + noStates);
    doKButton.setSelected(rule.getDoK());
    doLButton.setSelected(rule.getDoL());
    doMButton.setSelected(rule.getDoM());
    doNButton.setSelected(rule.getDoN());
  }

  public void getData(Primordia rule)
  {
    rule.setNoStates(noStates);
    rule.setDoK(doKButton.isSelected());
    rule.setDoL(doLButton.isSelected());
    rule.setDoM(doMButton.isSelected());
    rule.setDoN(doNButton.isSelected());
  }

  public boolean hasChanged()
  {
    return changed;
  }

  private void setValues()
  {
    try
    {
      int s = new Integer(noStatesInput.getText().trim());
      noStates = s;
      if (noStates < 2)
        noStates = 2;
      if (noStates > 18)
        noStates = 18;
    }
    catch (Exception e)
    {
    }
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
