import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class RandomOptionsDialog extends JDialog implements ActionListener, DocumentListener
{
  private SquareCell squareCell;

  private boolean clear;
  private int density;
  private boolean changed;

  private JLabel densityLabel;
  private JTextField densityInput;
  private ButtonGroup initOptionSelector;
  private JRadioButton clearButton;
  private JRadioButton overlayButton;
  private RuleTable minStateButton;
  private JLabel minStateLabel;
  private RuleTable maxStateButton;
  private JLabel maxStateLabel;
  private JCheckBox randomAll;
  private JButton okButton;
  private JButton cancelButton;

  public RandomOptionsDialog(JFrame owner, SquareCell squareCell)
  {
    super(owner, "Random", true);
    this.squareCell = squareCell;
    densityLabel = new JLabel("% Density");
    densityInput = new JTextField("" + density, 3);

    densityInput.getDocument().addDocumentListener(this);
    initOptionSelector = new ButtonGroup();
    clearButton = new JRadioButton("Clear");
    clearButton.addActionListener(this);
    initOptionSelector.add(clearButton);
    overlayButton = new JRadioButton("Overlay");
    overlayButton.addActionListener(this);
    initOptionSelector.add(overlayButton);
    clearButton.setSelected(true);

    minStateButton = new RuleTable(1, 1, squareCell, true);
    minStateLabel = new JLabel("Minimum");
    JPanel minStatePanel = new JPanel();
    minStatePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    minStatePanel.add(minStateButton);
    minStatePanel.add(minStateLabel);
    maxStateButton = new RuleTable(1, 1, squareCell, true);
    maxStateLabel = new JLabel("Maximum");
    JPanel maxStatePanel = new JPanel();
    maxStatePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    maxStatePanel.add(maxStateButton);
    maxStatePanel.add(maxStateLabel);
    randomAll = new JCheckBox("All");

    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();

    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel densityPanel = new JPanel();
    densityPanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel densityInputPanel = new JPanel();
    densityInputPanel.add(densityInput);
    densityPanel.add(densityLabel);
    densityPanel.add(densityInputPanel);
    densityPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ""));

    JPanel initOptionPanel = new JPanel();
    initOptionPanel.setLayout(new GridLayout(2, 1, 5, 5));
    initOptionPanel.add(clearButton);
    initOptionPanel.add(overlayButton);
    initOptionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Initialization Option"));

    JPanel noStatesPanel = new JPanel();
    noStatesPanel.setLayout(new GridLayout(3, 1, 5, 5));
    noStatesPanel.add(randomAll);
    noStatesPanel.add(minStatePanel);
    noStatesPanel.add(maxStatePanel);
    noStatesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "State Range"));

    JPanel optionsPanel = new JPanel();
    optionsPanel.setLayout(new BorderLayout(0, 5));
    optionsPanel.add(initOptionPanel, BorderLayout.NORTH);
    optionsPanel.add(noStatesPanel, BorderLayout.SOUTH);

    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);

    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(densityPanel, BorderLayout.NORTH);
    basePanel.add(optionsPanel, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    try
    {
      int s = new Integer(densityInput.getText().trim());
      density = s;
      if (density < 1)
        density = 1;
      if (density > 100)
        density = 100;
    }
    catch (Exception e)
    {
    }
  }

  public void setData(SquareCell squareCell)
  {
    density = squareCell.getRandomDensity();
    densityInput.setText("" + density);
    if (squareCell.getRandomClear())
      clearButton.setSelected(true);
    else
      overlayButton.setSelected(true);
    randomAll.setSelected(squareCell.getRandomAll());
    minStateButton.setState(0, 0, squareCell.getRandomMinState());
    maxStateButton.setState(0, 0, squareCell.getRandomMaxState());
  }

  public void getData(SquareCell squareCell)
  {
    squareCell.setRandomDensity(density);
    if (clearButton.isSelected())
      squareCell.setRandomClear(true);
    else
      squareCell.setRandomClear(false);
    squareCell.setRandomAll(randomAll.isSelected());
    squareCell.setRandomMinState(minStateButton.getState(0, 0));
    squareCell.setRandomMaxState(maxStateButton.getState(0, 0));
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == clearButton)
      clear = true;
    else if (source == overlayButton)
      clear = false;
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
