import java.awt.*;
import javax.swing.*;
import java.awt.image.BufferedImage;

public class SquareGrid extends JPanel
{
  private final int border = 4;
  
  private int worldRows;
  private int worldColumns;
  private int rows;
  private int columns;
  private int topBorder;
  private int leftBorder;
  private int squareSize;
  private boolean selected;
  private int grid[][];
  private BufferedImage image;
  private Selector selector;
  private ColorSelector colorSelector;

  public SquareGrid(int worldRows, int worldColumns, int squareSize, Selector selector, ColorSelector colorSelector)
  {
    this.worldRows = worldRows;
    this.worldColumns = worldColumns;
    this.squareSize = squareSize;
    this.selector = selector;
    this.colorSelector = colorSelector;
    selected = false;
    setBackground(Color.black);
    setOpaque(true);
    setDoubleBuffered(false);
  }

  public int getRows()
  {
    return rows;
  }

  public int getColumns()
  {
    return columns;
  }  

  public int xCoordToColumnNumber(int x)
  {
    int c = (x - leftBorder) / squareSize;
    if ((x < leftBorder) || (c >= columns))
      return -1;
    else
      return c;
  }
 
  public int yCoordToRowNumber(int y)
  {
    int r = (y - topBorder) / squareSize;
    if ((y < topBorder) || (r >= rows))
      return -1;
    else
      return r;
  }

  public void setSelected(boolean selected)
  {
    this.selected = selected;
  }

  public void setState(int r, int c, int state)
  {
    if (r >= 0 && r < rows && c >= 0 && c < columns)
    {
      grid[r][c] = state;
    }
  }

  public int getState(int r, int c)
  {
    return grid[r][c];
  }

  public void setSquareSize(int squareSize)
  {
    this.squareSize = squareSize;
    makeGrid();
  }

  public void makeGrid()
  {
    int height = getHeight();
    int width = getWidth();
    rows = (height - 2 * border) / squareSize;
    if (rows > worldRows)
      rows = worldRows;
    columns = (width - 2 * border) / squareSize;
    if (columns > worldColumns)
      columns = worldColumns;
    topBorder = (height - (rows * squareSize)) / 2;
    leftBorder = (width - (columns * squareSize)) / 2;
    grid = new int[rows][columns];
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    if ((image == null) || image.getWidth() != getWidth() || image.getHeight() != getHeight())
      image = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_RGB);
    drawSquares();
    if (selected)
      drawSelector();
    g.drawImage(image, 0, 0, null);
  }

  private void drawSquares()
  {
    Graphics g = image.getGraphics();
    g.clearRect(0, 0, image.getWidth(), image.getHeight());
    for (int r = 0; r < rows; r++)
      for (int c = 0; c < columns; c++)
        drawSquare(g, r, c, colorSelector.getColor(grid[r][c]));
    g.dispose();
  }

  private void drawSquare(Graphics g, int r, int c, Color color)
  {
    int y = topBorder + (r * squareSize);
    int h = squareSize;
    int x = leftBorder + (c * squareSize);
    int w = squareSize;
    if (squareSize > 5)
    { 
      g.setColor(colorSelector.getGridColor());
      g.drawRect(x, y, squareSize, squareSize);
      g.setColor(color);
      g.fillRect(x + 1, y + 1, squareSize - 1, squareSize - 1);
    }
    else if (squareSize > 1)
    { 
      g.setColor(color);
      g.fillRect(x, y, squareSize, squareSize);
    }
  }

  private void drawSelector()
  {
    Graphics g = image.getGraphics();
    if (selector.getState() == Selector.State.GroupSelected)
    {
      Color color;
      for (int r = 0; r < selector.getRows(); r++)
        for (int c = 0; c < selector.getColumns(); c++)
        {
          color = colorSelector.getColor(selector.getCellState(r, c));
          if (color != colorSelector.getColor(0))
            drawSquare(g, selector.getTopLeftRow() + r, selector.getTopLeftColumn() + c, color);
        }
    }
    g.setColor(colorSelector.getSelectorColor());
    int x1 = leftBorder + (selector.getTopLeftColumn() * squareSize);
    int y1 = topBorder + (selector.getTopLeftRow() * squareSize);
    int x2 = leftBorder + (selector.getBottomRightColumn() * squareSize);
    int y2 = topBorder + (selector.getBottomRightRow() * squareSize);
    g.drawRect(x1, y1, x2 - x1 + squareSize, y2 - y1 + squareSize);
    g.dispose();
  }
}
